package main

import (
	"flag"
	"fmt"
	"net/http"
	"net/http/pprof"
	"pool/pool"
	"runtime"
	"strings"
)

//var addr = flag.String("12", ":8081", "http service address")

func serveHome(w http.ResponseWriter, r *http.Request) {
	fmt.Println(r.URL)
	if r.URL.Path != "/" {
		http.Error(w, "Not found", http.StatusNotFound)
		return
	}
	if r.Method != "GET" {
		http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		return
	}
	http.ServeFile(w, r, "home.html")
}

var ch = make(chan int, 10)

func chfun(i int) {
	fmt.Println("写入管道i的值%d", i)
	//ch<-i
	select {
	case ch <- i:
		return
	default:
		fmt.Println("管道己经锁定；i的值" + string(i))
	}
}

func main() {
	runtime.GOMAXPROCS(runtime.NumCPU())

	/*	for i:=0;i<10000 ;i++  {
			go chfun(i)
		}
		close(ch)
	*/

	/*for {
		select {
		case i,ok:=<-ch:
			if !ok{
				log.Println("管道己经关闭%d",i)
			}
			log.Println("读取i的值%d",i)

		}
	}
	*/

	flag.Parse()
	//初骀化连接池
	pool.InitWsPool(func(err interface{}) {
		//接收连接池中的运行时错误信息
		fmt.Println("wsPool.InitWsPool error-------------", err)
	})

	mux := http.NewServeMux()
	mux.HandleFunc("/", serveHome)

	mux.HandleFunc("/debug/pprof/", pprof.Index)
	mux.HandleFunc("/debug/pprof/cmdline", pprof.Cmdline)
	mux.HandleFunc("/debug/pprof/profile", pprof.Profile)
	mux.HandleFunc("/debug/pprof/symbol", pprof.Symbol)
	mux.HandleFunc("/debug/pprof/trace", pprof.Trace)

	mux.HandleFunc("/ws", ws)
	err := http.ListenAndServe(":11001", mux)
	if err != nil {
		fmt.Printf("ListenAndServe: %s", err.Error())
	}
}

func ws(w http.ResponseWriter, r *http.Request) {
	headData := r.Header.Get("Sec-Websocket-Protocol")
	list := strings.Split(headData, "-")
	head := http.Header{}
	head.Add("Sec-Websocket-Protocol", headData)

	//实例化连接对象
	client := pool.NewClient(&pool.Config{
		Id:        list[0],  //连接标识
		Type:      "ws",     //连接类型
		Channel:   list[1:], //指定频道
		Goroutine: 100,
	})
	fmt.Println(client.Id, "实例化连接对象完成")

	//连接成功回调
	client.OnOpen(func() {
		fmt.Printf("连接己开启%s", client.Id)
	})

	//接收消息
	client.OnMessage(func(msg *pool.SendMsg) {
		if msg.Status == 3 {
			fmt.Println("OnMessage:收到出错消息=》", client.Id, msg.Desc)
			return
		}
		fmt.Println(msg.Msg)
		if msg.ToClientId != "" {
			//发送消息给指定的ToClientID连接
			err := pool.Send(msg)
			if err != nil {
				fmt.Println("wsPool.Send(msg):", err.Error())
			}
			//发送消息给当前连接对象
			//err = client.Send(msg)
			//if err != nil {
			//	fmt.Println("client.Send(msg):", err.Error())
			//}
		}
		/*if len(msg.Channel)>0{
			//按频道广播，可指定多个频道[]string
			err:=wsPool.Broadcast(msg) //或者 wsPool.Broadcast(msg)
			if err!=nil {
				log.Println("wsPool.Broadcast(msg)", err.Error())
			}
		}
		//或都全局广播，所有连接都进行发送
		err:=wsPool.BroadcastAll(msg)
		if err!=nil {
			log.Println("wsPool.BroadcastAll(msg)", err.Error())
		}*/

	})
	//连接断开回调
	client.OnClose(func() {
		fmt.Printf("连接己经关闭%s", client.Id)
	})
	client.OnError(func(err error) {
		fmt.Printf("连接%s错误信息：%s", client.Id, err.Error())
	})

	//开启连接
	client.OpenClient(w, r, head)
	fmt.Println(client.Id, "开启连接")

	r.Close = true
	return
}
